/* FOC Demo Application, tested with:
 *      MCLV2
 *      Hurst Brushless
 *      dsPIC33EP256MC506
 */

#ifndef __HARDWARE_H__
#define __HARDWARE_H__

#include <p33Exxxx.h>
#include "SerialGeneric.h"

/* public prototypes */
void initHardware(void);
void initSerial(tSerial* serial);


// alter for different PWM-frequency
//#define PWM15KHZ

#define PWMFREQUENCY 20000
#ifdef PWM10KHZ
    #undef PWMFREQUENCY
    #define PWMFREQUENCY	10000		// PWM Frequency in Hertz
#endif
#ifdef PWM15KHZ
    #undef PWMFREQUENCY
    #define PWMFREQUENCY    15000
#endif

#define PULSE_PER_REV 1000 // ENCODER Pulses
#define N_P     5            // Electrical Pole-Pairs

#define DEADTIMESEC     0.000001    // Dead time in seconds - 1us
#define DFCY            70000000		// Instruction cycle frequency (Hz)
#define DTCY            (1.0/DFCY)		// Instruction cycle period (sec)
#define DDEADTIME       (unsigned int)(DEADTIMESEC*DFCY)	// Dead time in dTcys
#define MIN_DUTY        (unsigned int)(DDEADTIME/2 + 1)        // Should be >= DDEADTIME/2 for PWM Errata workaround
#define LOOPINTCY	DFCY/PWMFREQUENCY   // Basic loop period in units of Tcy

#define PWM_MIN     0.1 * LOOPINTCY
#define PWM_MAX     0.9 * LOOPINTCY

/* public macros */
#define KICK_DOG \
    do { \
        ClrWdt(); \
    } while (0)

#endif