/*
 * Copyright (c) {YEAR}, {ORGANIZATION_NAME}, {ORGANIZATION_WEB}
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the "{ORGANIZATION_NAME}" nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "{ORGANIZATION_NAME}" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://www.mechatronic-simulation.org/
 * $LastChangedRevision$
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "clark_INT16_ASM.h"

/* all used update functions to ram for c2000                                                                         */
#if defined(__ALL_UPDATE_FUNC_2_RAM_C2000__)
     #pragma CODE_SECTION(clark_INT16_ASM_Update, "ramfuncs")
#endif

/* USERCODE-BEGIN:PreProcessor                                                                                        */
#include "motor_control.h"
#define A 	  	*pTclark_INT16_ASM->a
#define B		*pTclark_INT16_ASM->b

#define ALPHA 	pTclark_INT16_ASM->alpha  
#define BETA 	pTclark_INT16_ASM->beta                                                                                
/* USERCODE-END:PreProcessor                                                                                          */

/**********************************************************************************************************************/
/** Update                                                                                                           **/
/**********************************************************************************************************************/
void clark_INT16_ASM_Update(CLARK_INT16_ASM *pTclark_INT16_ASM)
{
/* USERCODE-BEGIN:UpdateFnc                                                                                           */
    
#ifdef __XC16__
    MC_ABC_T mcIabc;
    MC_ALPHABETA_T mcIAlphaBeta;
    
    mcIabc.a = A;
    mcIabc.b = B;
    mcIabc.c = -(A+B);
    
    MC_TransformClarke_Assembly(&mcIabc, &mcIAlphaBeta);
    
    ALPHA   = mcIAlphaBeta.alpha;
    BETA    = mcIAlphaBeta.beta;
#else
	int32 temp;
	temp = B;
	temp <<= 1;   // B * 2
	temp += A; // A + 2*B
	temp = temp * 18907;
	temp >>= 15; // (A+2*B) * 0.577
	if (temp > INT16_MAX)
	{
		temp = INT16_MAX;
	}
	else
	{
		if (temp < -INT16_MAX)
		{
			temp = -INT16_MAX;
		}
	}
	BETA = (int16)temp;
	ALPHA = A;
#endif
/* USERCODE-END:UpdateFnc                                                                                             */

}

/**********************************************************************************************************************/
/** Initialization                                                                                                   **/
/**********************************************************************************************************************/
void clark_INT16_ASM_Init(CLARK_INT16_ASM *pTclark_INT16_ASM)
{
     pTclark_INT16_ASM->ID = CLARK_INT16_ASM_ID;
     pTclark_INT16_ASM->alpha = 0;
     pTclark_INT16_ASM->beta = 0;
/* USERCODE-BEGIN:InitFnc                                                                                             */
/* USERCODE-END:InitFnc                                                                                               */
}

/**********************************************************************************************************************/
/** No Load function                                                                                                 **/
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/** No Save function                                                                                                 **/
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/** Get block element address                                                                                        **/
/**********************************************************************************************************************/
#if !defined(CLARK_INT16_ASM_ISLINKED)
void* clark_INT16_ASM_GetAddress(const CLARK_INT16_ASM* block, uint16 elementId)
{
    void* addr;
    switch (elementId)
    {
        case 1:
            addr = (void*)block->a;
            break;
        case 2:
            addr = (void*)block->b;
            break;
        case 3:
            addr = (void*)&block->alpha;
            break;
        case 4:
            addr = (void*)&block->beta;
            break;
        default:
            addr = (void*)0;
            break;
    }
    return (addr);
}
#endif
