/*
 * Copyright (c) {YEAR}, {ORGANIZATION_NAME}, {ORGANIZATION_WEB}
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the "{ORGANIZATION_NAME}" nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "{ORGANIZATION_NAME}" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://www.mechatronic-simulation.org/
 * $LastChangedRevision$
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "clark_inv_INT16_ASM.h"

/* all used update functions to ram for c2000                                                                         */
#if defined(__ALL_UPDATE_FUNC_2_RAM_C2000__)
     #pragma CODE_SECTION(clark_inv_INT16_ASM_Update, "ramfuncs")
#endif

/* USERCODE-BEGIN:PreProcessor                                                                                        */
#include "motor_control.h"
#define ALPHA 			*pTclark_inv_INT16_ASM->alpha
#define BETA			*pTclark_inv_INT16_ASM->beta

#define A				pTclark_inv_INT16_ASM->a
#define B				pTclark_inv_INT16_ASM->b
#define C 				pTclark_inv_INT16_ASM->c
/* USERCODE-END:PreProcessor                                                                                          */

/**********************************************************************************************************************/
/** Update                                                                                                           **/
/**********************************************************************************************************************/
void clark_inv_INT16_ASM_Update(CLARK_INV_INT16_ASM *pTclark_inv_INT16_ASM)
{
/* USERCODE-BEGIN:UpdateFnc                                                                                           */
#ifdef __XC16__
    MC_ALPHABETA_T mcVAlphaBeta;    
    MC_ABC_T mcVabc;
    
    mcVAlphaBeta.alpha = BETA;
    mcVAlphaBeta.beta  = ALPHA;
    MC_TransformClarkeInverseSwappedInput_Assembly(&mcVAlphaBeta, &mcVabc);
    
    A = mcVabc.a;
    B = mcVabc.b;
    C = mcVabc.c;
#else 
	A = ALPHA;
	int32 temp1;
	int32 temp2;
	int32 temp3;
	
	temp1 = ALPHA;
	temp1 >>= 1; 					// alpha / 2
	
	temp2 = (int16)(((int32)BETA * 28377)>>15); // beta * 0.866	
	
	temp3 = (temp2 - temp1);
	if (temp3 > INT16_MAX)
	{
		temp3 = INT16_MAX;
	}
	else
	{
		if (temp3 < -INT16_MAX)
		{
			temp3 = -INT16_MAX;
		}
	}
	B = (int16)temp3;
	
	temp3 = (-temp1 - temp2);
	if (temp3 > INT16_MAX)
	{
		temp3 = INT16_MAX;
	}
	else
	{
		if (temp3 < -INT16_MAX)
		{
			temp3 = -INT16_MAX;
		}
	}
	C = (int16)temp3;
#endif
/* USERCODE-END:UpdateFnc                                                                                             */

}

/**********************************************************************************************************************/
/** Initialization                                                                                                   **/
/**********************************************************************************************************************/
void clark_inv_INT16_ASM_Init(CLARK_INV_INT16_ASM *pTclark_inv_INT16_ASM)
{
     pTclark_inv_INT16_ASM->ID = CLARK_INV_INT16_ASM_ID;
     pTclark_inv_INT16_ASM->a = 0;
     pTclark_inv_INT16_ASM->b = 0;
     pTclark_inv_INT16_ASM->c = 0;
/* USERCODE-BEGIN:InitFnc                                                                                             */
/* USERCODE-END:InitFnc                                                                                               */
}

/**********************************************************************************************************************/
/** No Load function                                                                                                 **/
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/** No Save function                                                                                                 **/
/**********************************************************************************************************************/

/**********************************************************************************************************************/
/** Get block element address                                                                                        **/
/**********************************************************************************************************************/
#if !defined(CLARK_INV_INT16_ASM_ISLINKED)
void* clark_inv_INT16_ASM_GetAddress(const CLARK_INV_INT16_ASM* block, uint16 elementId)
{
    void* addr;
    switch (elementId)
    {
        case 1:
            addr = (void*)block->alpha;
            break;
        case 2:
            addr = (void*)block->beta;
            break;
        case 3:
            addr = (void*)&block->a;
            break;
        case 4:
            addr = (void*)&block->b;
            break;
        case 5:
            addr = (void*)&block->c;
            break;
        default:
            addr = (void*)0;
            break;
    }
    return (addr);
}
#endif
