/*
 * Copyright (c) {YEAR}, {ORGANIZATION_NAME}, {ORGANIZATION_WEB}
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the "{ORGANIZATION_NAME}" nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "{ORGANIZATION_NAME}" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://www.mechatronic-simulation.org/
 * $LastChangedRevision$
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "sawtooth_INT16.h"

/* all used update functions to ram for c2000                                                                         */
#if defined(__ALL_UPDATE_FUNC_2_RAM_C2000__)
     #pragma CODE_SECTION(sawtooth_INT16_Update, "ramfuncs")
#endif

/* USERCODE-BEGIN:PreProcessor                                                                                        */
#define OUT pTsawtooth_INT16->out           
#define DIR pTsawtooth_INT16->cnt_dir 
#define LIM pTsawtooth_INT16->time_ctr
#define CNT pTsawtooth_INT16->cnt_val      
#define ADD pTsawtooth_INT16->add        
#define ENABLE *pTsawtooth_INT16->enable  
/* USERCODE-END:PreProcessor                                                                                          */

/**********************************************************************************************************************/
/** Update                                                                                                           **/
/**********************************************************************************************************************/
void sawtooth_INT16_Update(SAWTOOTH_INT16 *pTsawtooth_INT16)
{
/* USERCODE-BEGIN:UpdateFnc                                                                                           */
		int32 temp;
		if (ENABLE != 0) {
			if(CNT >= LIM) {
				DIR = 0;
			}
			else if (CNT <= 0) {
				DIR = INT16_MAX;
			}
		
			if(DIR != 0) {
				temp = OUT + ADD;
				CNT += 1;
			}
			else {
				temp = OUT - ADD;
				CNT -= 1;
			}
		}
		else {
			temp = 0;
		}
		
		if (temp > INT16_MAX)
			temp = INT16_MAX;
		
		OUT = (int16)temp;

/* USERCODE-END:UpdateFnc                                                                                             */

}

/**********************************************************************************************************************/
/** Initialization                                                                                                   **/
/**********************************************************************************************************************/
void sawtooth_INT16_Init(SAWTOOTH_INT16 *pTsawtooth_INT16)
{
     pTsawtooth_INT16->ID = SAWTOOTH_INT16_ID;
     pTsawtooth_INT16->out = 0;
/* USERCODE-BEGIN:InitFnc                                                                                             */
	 DIR = INT16_MAX;
	 CNT = 0;
/* USERCODE-END:InitFnc                                                                                               */
}

/**********************************************************************************************************************/
/** Load block data                                                                                                  **/
/**********************************************************************************************************************/
uint8 sawtooth_INT16_Load(const SAWTOOTH_INT16 *pTsawtooth_INT16, uint8 data[])
{
     uint8 loadSize;

     data[0] = (uint8)(pTsawtooth_INT16->time_ctr & 0x00FF);
     data[1] = (uint8)((pTsawtooth_INT16->time_ctr >> 8) & 0x00FF);
     data[2] = (uint8)(pTsawtooth_INT16->cnt_dir & 0x00FF);
     data[3] = (uint8)((pTsawtooth_INT16->cnt_dir >> 8) & 0x00FF);
     data[4] = (uint8)(pTsawtooth_INT16->add & 0x00FF);
     data[5] = (uint8)((pTsawtooth_INT16->add >> 8) & 0x00FF);
     data[6] = (uint8)(pTsawtooth_INT16->cnt_val & 0x00FF);
     data[7] = (uint8)((pTsawtooth_INT16->cnt_val >> 8) & 0x00FF);
     loadSize = (uint8)8;
/* USERCODE-BEGIN:LoadFnc                                                                                             */
/* USERCODE-END:LoadFnc                                                                                               */
     return (loadSize);
}

/**********************************************************************************************************************/
/** Save block data                                                                                                  **/
/**********************************************************************************************************************/
uint8 sawtooth_INT16_Save(SAWTOOTH_INT16 *pTsawtooth_INT16, const uint8 data[], 
          uint8 frameLength)
{
     uint8 error;

     if (frameLength != (uint8)8)
     {
          error = (uint8)1;
     }
     else
     {
          pTsawtooth_INT16->time_ctr = UINT16_TO_INT16((uint16)data[0] + \
               ((uint16)data[1] << 8));
          pTsawtooth_INT16->cnt_dir = UINT16_TO_INT16((uint16)data[2] + \
               ((uint16)data[3] << 8));
          pTsawtooth_INT16->add = UINT16_TO_INT16((uint16)data[4] + \
               ((uint16)data[5] << 8));
          pTsawtooth_INT16->cnt_val = UINT16_TO_INT16((uint16)data[6] + \
               ((uint16)data[7] << 8));
          error = (uint8)0;
/* USERCODE-BEGIN:SaveFnc                                                                                             */
/* USERCODE-END:SaveFnc                                                                                               */
     }
     return (error);
}

/**********************************************************************************************************************/
/** Get block element address                                                                                        **/
/**********************************************************************************************************************/
#if !defined(SAWTOOTH_INT16_ISLINKED)
void* sawtooth_INT16_GetAddress(const SAWTOOTH_INT16* block, uint16 elementId)
{
    void* addr;
    switch (elementId)
    {
        case 1:
            addr = (void*)block->enable;
            break;
        case 2:
            addr = (void*)&block->out;
            break;
        default:
            addr = (void*)0;
            break;
    }
    return (addr);
}
#endif
